unit fODMedIV;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  fODBase, Grids, StdCtrls, ORCtrls, ComCtrls, ExtCtrls, ORDtTm,ORFn;

type
  TfrmODMedIV = class(TfrmODBase)
    lblInfusionRate: TLabel;
    txtRate: TEdit;
    lblComponent: TLabel;
    lblAmount: TLabel;
    lblComments: TLabel;
    memComments: TMemo;
    txtSelected: TEdit;
    cboSelected: TComboBox;
    Label1: TLabel;
    grdSelected: TStringGrid;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cmdRemoveClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure grdSelectedSelectCell(Sender: TObject; Col, Row: Integer; var CanSelect: Boolean);
    procedure txtSelectedExit(Sender: TObject);
    procedure cboSelectedExit(Sender: TObject);
    procedure ControlChange(Sender: TObject);
    procedure txtSelectedChange(Sender: TObject);
    procedure cboSelectedChange(Sender: TObject);
    procedure grdSelectedDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
  private
    FNow: TFMDateTime;
    FInpatient: boolean;
    FInit: boolean;
    function IsLastOne(const OrderableIEN: integer): boolean;
    procedure SetOrderBox(const ASolutionIDs: TStringList; const AnAdditiveIDs: TStringList;
                  theSolutions: TStringList; theAdditives: TStringList);
  public
    procedure InitDialog; override;
    procedure SetupDialog(OrderAction: Integer; const ID: string); override;
    procedure Validate(var AnErrMsg: string); override;
  end;

const
  TX_NO_DEA     = 'Provider must have a DEA# or VA# to order this medication';
  TC_NO_DEA     = 'DEA# Required';

var
  frmODMedIV: TfrmODMedIV;

implementation

{$R *.DFM}

uses rODBase, uConst, rCore, uCore, rODMeds;

type
  TIVComponent = class
  private
    IEN: Integer;
    Name: string;
    Fluid: Char;
    Amount: Integer;
    Units: string;
    Volumes: string;
    PharmType: string;
  end;

const
  TC_RESTRICT  = 'Ordering Restrictions';
  TX_NO_BASE   = 'A solution must be selected.';
  TX_NO_AMOUNT = 'A strength or volume must be entered for ';
  TX_NO_UNITS  = 'Units must be entered for ';
  TX_NO_RATE   = 'An infusion rate must be entered.';
  TX_BAD_RATE  = 'The infusion rate must be:  # ml/hr  or  text@labels per day';

{ Form methods }

procedure TfrmODMedIV.FormCreate(Sender: TObject);
var
  Restriction: string;
begin
  inherited;
  AllowQuickOrder := False;
  CheckAuthForMeds(Restriction);
  if Length(Restriction) > 0 then
  begin
    InfoBox(Restriction, TC_RESTRICT, MB_OK);
    Close;
    Exit;
  end;
  FillerID := 'PSIV';
  StatusText('Loading Dialog Definition');
  Responses.Dialog := 'PSJI OR PAT FLUID OE';
  StatusText('Loading Default Values');
  CtrlInits.LoadDefaults(ODForIVFluids);
  InitDialog;
end;

procedure TfrmODMedIV.FormDestroy(Sender: TObject);
var
  i: Integer;
begin
  inherited;
  with grdSelected do for i := 0 to RowCount - 1 do TIVComponent(Objects[0, i]).Free;
  if BCMASolutions.Count > 0 then
    BCMASolutions.Clear;
  if BCMAAdditives.Count > 0 then
    BCMAAdditives.Clear;
  BCMASolutions := nil;
  BCMAAdditives := nil;
end;

procedure TfrmODMedIV.FormResize(Sender: TObject);
begin
  inherited;
  with grdSelected do
  begin
    ColWidths[1] := Canvas.TextWidth(' 10000 ') + GetSystemMetrics(SM_CXVSCROLL);
    ColWidths[2] := Canvas.TextWidth('meq.') + GetSystemMetrics(SM_CXVSCROLL);
    ColWidths[0] := ClientWidth - ColWidths[1] - ColWidths[2] - 2;
  end;
end;

procedure TfrmODMedIV.InitDialog;
const
  NOSELECTION: TGridRect = (Left: -1; Top: -1; Right: -1; Bottom: -1);
var
  i: Integer;
  theSolutions: TStringList;
  theAdditives: TStringList;
begin
  inherited;
  theSolutions := TStringList.Create;
  theAdditives := TStringList.Create;
  FInit := True;
  with grdSelected do for i := 0 to RowCount - 1 do
  begin
     TIVComponent(Objects[0, i]).Free;
     Rows[i].Clear;
  end;
  grdSelected.RowCount := 1;
  if BCMASolutions.Count > 0 then
  begin
    theSolutions.Assign(SubSetOfBCMAOrderItems(BCMASolutions, 'S.IVB RX'));
  end;
  if BCMAAdditives.Count > 0 then
  begin
    theAdditives.Assign(SubSetOfBCMAOrderItems(BCMAAdditives, 'S.IVA RX'));
  end;
  SetOrderBox(BCMASolutions, BCMAAdditives, theSolutions, theAdditives);
  StatusText('');
  if BCMASolutions.Count > 0 then
    BCMASolutions.Clear;
  if BCMAAdditives.Count > 0 then
    BCMAAdditives.Clear;
  FNow  := FMNow;
  FInit := False;
  cboSelected.SendToBack;
  ControlChange(Self);
end;

procedure TfrmODMedIV.Validate(var AnErrMsg: string);
var
  ItemOK: Boolean;
  x: string;
  i: Integer;

  procedure SetError(const x: string);
  begin
    if Length(AnErrMsg) > 0 then AnErrMsg := AnErrMsg + CRLF;
    AnErrMsg := AnErrMsg + x;
  end;

begin
  inherited;
  with grdSelected do
  begin
    ItemOK := False;
    for i := 0 to RowCount - 1 do
      if (Objects[0,i]<> nil) and (TIVComponent(Objects[0, i]).Fluid = 'B') then ItemOK := True;
    if not ItemOK then SetError(TX_NO_BASE);
    for i := 0 to RowCount - 1 do
    begin
      if (Objects[0, i] <> nil) and (Length(Cells[1, i]) = 0)
        then SetError(TX_NO_AMOUNT + Cells[0, i]);
      if (Objects[0, i] <> nil) and (Length(Cells[2, i]) = 0)
        then SetError(TX_NO_UNITS + Cells[0, i]);
    end;
  end;
  if Length(txtRate.Text) = 0 then SetError(TX_NO_RATE) else
  begin
    x := Trim(txtRate.Text);
    ValidateIVRate(x);
    if Length(x) = 0 then SetError(TX_BAD_RATE) else Responses.Update('RATE', 1, x, x);
  end;
end;

procedure TfrmODMedIV.SetOrderBox(const ASolutionIDs: TStringList; const AnAdditiveIDs: TStringList;
                  theSolutions: TStringList; theAdditives: TStringList);
var
  i,j: integer;
  AnIVComponent: TIVComponent;
  x,ATempSolution,thePharmType,dfltVolume: string;
begin
  thePharmType := '';
  dfltVolume   := '';
  for i := 0 to ASolutionIDs.Count - 1 do
  begin
    for j := 0 to theSolutions.Count - 1 do
    begin
      ATempSolution := Piece(theSolutions[j],'^',1);
      if ATempSolution = Piece(ASolutionIDs[i],';',1) then
      begin
        if Piece(theSolutions[j],'^',4) = 'NF' then
        begin
          if Length(Piece(ATempSolution,';',2))>0 then
          begin
            x := AmountsForIVFluidNF(StrToInt(Piece(ATempSolution,';',1)), 'A', 'NF');
            thePharmType := Piece(ATempSolution,';',2);
          end else
            x := AmountsForIVFluidNF(StrToInt(ATempSolution), 'B', 'NF');
        end
        else
          x := AmountsForIVFluid(StrToInt(Piece(ASolutionIDs[i],';',1)), 'B');
        if Pos(Piece(ASolutionIDs[i],';',2),x)=0 then
          x := x + '^' + Piece(ASolutionIDs[i],';',2);
        dfltVolume := Piece(ASolutionIDs[i],';',2);
        AnIVComponent := TIVComponent.Create;
        AnIVComponent.IEN       := StrToInt(Piece(ASolutionIDS[i],';',1));
        AnIVComponent.Name      := Piece(theSolutions[j], U, 3);
        AnIVComponent.Fluid     := 'B';
        if Length(dfltVolume)>0 then
          AnIVComponent.Amount    := StrToIntDef(dfltVolume, 0)
        else
          AnIVComponent.Amount    := StrToIntDef(Piece(x, U, 2), 0);
        AnIVComponent.Units     := Piece(x, U, 1);
        AnIVComponent.Volumes   := Copy(x, Pos(U, x) + 1, Length(x));
        AnIvComponent.PharmType := thePharmType;
        thePharmType := '';
        dfltVolume   := '';
        with grdSelected do
        begin
          if Objects[0, RowCount - 1] <> nil then RowCount := RowCount + 1;
          Objects[0, RowCount - 1] := AnIVComponent;
          Cells[0, RowCount - 1] := AnIVComponent.Name;
          Cells[1, RowCount - 1] := IntToStr(AnIVComponent.Amount);
          Cells[2, RowCount - 1] := AnIVComponent.Units;
          Row := RowCount - 1;
          if Length(Piece(AnIVComponent.Volumes, U, 2)) > 0 then Col := 1 else Col := 0;
        end;
        break;
      end;
    end;
  end;

  for i := 0 to AnAdditiveIDs.Count - 1 do
  begin
    for j := 0 to theAdditives.Count - 1 do
    begin
      if Piece(theAdditives[j],'^',1) = AnAdditiveIDs[i] then
      begin
        FInpatient := OrderForInpatient;
        if DEACheckFailed(StrToInt(AnAdditiveIDs[i]), FInpatient) then
        begin
           InfoBox(TX_NO_DEA, TC_NO_DEA, MB_OK);
           Exit;
        end;
        if Piece(theAdditives[j],'^',4) = 'NF' then
          x := AmountsForIVFluidNF(StrToInt(AnAdditiveIDs[i]), 'A', 'NF')
        else
          x := AmountsForIVFluid(StrToInt(AnAdditiveIDs[i]), 'A');
        AnIVComponent := TIVComponent.Create;
        AnIVComponent.IEN     := StrToInt(AnAdditiveIDs[i]);
        AnIVComponent.Name    := Piece(theAdditives[j], U, 3);
        AnIVComponent.Fluid   := 'A';
        AnIVComponent.Amount  := 0;
        AnIVComponent.Units   := Piece(x, U, 1);
        AnIVComponent.Volumes := '';
        with grdSelected do
        begin
          if Objects[0, RowCount - 1] <> nil then RowCount := RowCount + 1;
          Objects[0, RowCount - 1] := AnIVComponent;
          Cells[0, RowCount - 1] := AnIVComponent.Name;
          Cells[2, RowCount - 1] := AnIVComponent.Units;
          Row := RowCount - 1;
          Col := 1;
        end;
        break;
      end;
    end;
  end;
end;

procedure TfrmODMedIV.SetupDialog(OrderAction: Integer; const ID: string);
begin
  inherited;
end;

procedure TfrmODMedIV.grdSelectedSelectCell(Sender: TObject; Col, Row: Integer;
  var CanSelect: Boolean);
var
  AnIVComponent: TIVComponent;

  procedure PlaceControl(AControl: TWinControl);
  var
    ARect: TRect;
  begin
    with AControl do
    begin
      ARect := grdSelected.CellRect(Col, Row);
      SetBounds(ARect.Left + grdSelected.Left + 1,  ARect.Top  + grdSelected.Top + 1,
                ARect.Right - ARect.Left + 1,       ARect.Bottom - ARect.Top + 1);
      BringToFront;
      Show;
      SetFocus;
    end;
  end;

begin
  inherited;
  AnIVComponent := TIVComponent(grdSelected.Objects[0, Row]);
  if (AnIVComponent = nil) or (Col = 0) then Exit;
  if (Col = 2) and (Length(Piece(AnIVComponent.Units, U, 2)) > 0) then
  begin
    PiecesToList(AnIVComponent.Units, U, cboSelected.Items);
    cboSelected.ItemIndex := cboSelected.Items.IndexOf(grdSelected.Cells[Col, Row]);
    cboSelected.Tag  := (Col * 256) + Row;
    if not FInit then
      PlaceControl(cboSelected);
  end;

  if (Col = 1) and (Length(Piece(AnIVComponent.Volumes, U, 2)) > 0) and (AnIVComponent.PharmType = '') then
  begin
    PiecesToList(AnIVComponent.Volumes, U, cboSelected.Items);
    cboSelected.ItemIndex := cboSelected.Items.IndexOf(grdSelected.Cells[Col, Row]);
    cboSelected.Tag  := (Col * 256) + Row;
    if not FInit then
      PlaceControl(cboSelected);
  end;

  if (Col = 1) and (Length(Piece(AnIVComponent.Volumes, U, 2)) = 0) and (AnIVComponent.Fluid = 'B') and (AnIVComponent.PharmType = '') then
  begin
    txtSelected.Text := grdSelected.Cells[Col,Row];
    txtSelected.Tag  := (Col * 256) + Row;
    if not FInit then
      PlaceControl(txtSelected);
  end;

  if (Col = 1) and (AnIVComponent.Fluid = 'A') then
  begin
    txtSelected.Text := grdSelected.Cells[Col, Row];
    txtSelected.Tag  := (Col * 256) + Row;
    if not FInit then
      PlaceControl(txtSelected);
  end;
end;

procedure TfrmODMedIV.txtSelectedChange(Sender: TObject);
begin
  inherited;
  with txtSelected do
  begin
    if Tag < 0 then Exit;
    grdSelected.Cells[Tag div 256, Tag mod 256] := Text;
  end;
  ControlChange(Sender);
end;

procedure TfrmODMedIV.txtSelectedExit(Sender: TObject);
begin
  inherited;
  with txtSelected do
  begin
    grdSelected.Cells[Tag div 256, Tag mod 256] := Text;
    Tag := -1;
    Hide;
  end;
end;

procedure TfrmODMedIV.cboSelectedChange(Sender: TObject);  
begin
  inherited;
  with cboSelected do
  begin
    if Tag < 0 then Exit;
    grdSelected.Cells[Tag div 256, Tag mod 256] := Text;
  end;
  ControlChange(Sender);
end;

procedure TfrmODMedIV.cboSelectedExit(Sender: TObject);
var
 V,Code: integer;
begin
  inherited;
  with cboSelected do
  begin
  try
    begin
      grdSelected.Cells[Tag div 256, Tag mod 256] := Text;
      Tag := -1;
      Hide;
    end;
  except
    //doing nothing
  end;
  end;
  cboSelected.SendToBack;
end;

procedure TfrmODMedIV.cmdRemoveClick(Sender: TObject);
var
  i: Integer;
  theIEN: integer;
begin
  inherited;
  theIEN := 0;
  with grdSelected do
  begin
    if Row < 0 then Exit;
    if Objects[0, Row] <> nil then
    begin
     theIEN := TIVComponent(Objects[0, Row]).IEN;
     if IsLastOne(theIEN) then
       Exit;
     TIVComponent(Objects[0, Row]).Free;
    end;
    for i := Row to RowCount - 2 do Rows[i] := Rows[i + 1];
    Rows[RowCount - 1].Clear;
    RowCount := RowCount - 1;
  end;
  ControlChange(Sender);
end;

{ update Responses & Create Order Text }

procedure TfrmODMedIV.ControlChange(Sender: TObject);
var
  i, CurAdd, CurBase: Integer;
  x: string;
  AnIVComponent: TIVComponent;

  function IsNumericRate(const x: string): Boolean;
  var
    i: Integer;
  begin
    Result := True;
    for i := 1 to Length(x) do if not (x[i] in ['0'..'9','.']) then Result := False;
  end;

begin
  inherited;
  if Changing then Exit;
  Responses.Clear;
  CurAdd := 1; CurBase := 1;
  with grdSelected do for i := 0 to RowCount - 1 do
  begin
    AnIVComponent := TIVComponent(Objects[0, i]);
    if AnIVComponent = nil then Continue;
    with AnIVComponent do
    begin
      if Fluid = 'B' then
      begin
        if IEN > 0                then Responses.Update('ORDERABLE', CurBase, IntToStr(IEN), Name);
        if Length(Cells[1,i]) > 0 then Responses.Update('VOLUME',    CurBase, Cells[1,i], Cells[1,i]);
        Inc(CurBase);
      end;
      if Fluid = 'A' then
      begin
        if IEN > 0                then Responses.Update('ADDITIVE', CurAdd, IntToStr(IEN), Name);
        if Length(Cells[1,i]) > 0 then Responses.Update('STRENGTH', CurAdd, Cells[1,i], Cells[1,i]);
        if Length(Cells[2,i]) > 0 then Responses.Update('UNITS',    CurAdd, Cells[2,i], Cells[2,i]);
        Inc(CurAdd);
      end;
    end;
  end;
  x := txtRate.Text;
  if IsNumericRate(x) then x := x + ' ml/hr';
  if (Pos('@',x)>0) and (Piece(x,'@',1) = IntToStr(StrToIntDef(Piece(x,'@',1), -1)))
    then x := Piece(x,'@',1) + ' ml/hr@' + Copy(x, Pos('@',x) + 1, Length(x));
  with txtRate     do if (Length(Text) > 0) then Responses.Update('RATE', 1, x, x);
  Responses.Update('URGENCY', 1, '99', '');
  with memComments do if GetTextLen > 0     then Responses.Update('COMMENT', 1, TX_WPTYPE, Text);
  memOrder.Text := Responses.OrderText;
end;

procedure TfrmODMedIV.grdSelectedDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
  State: TGridDrawState);
begin
  inherited;
  if Sender = ActiveControl then Exit;
  if not (gdSelected in State) then Exit;
  with Sender as TStringGrid do
  begin
    Canvas.Brush.Color := Color;
    Canvas.Font := Font;
    Canvas.TextRect(Rect, Rect.Left + 2, Rect.Top + 2, Cells[ACol, ARow]);
  end;
end;

function TfrmODMedIV.IsLastOne(const OrderableIEN: integer): boolean;
var
  i,cnt: integer;
  LastOne: Boolean;
begin
  LastOne := True;
  cnt := 0;
  for i := 0 to grdSelected.RowCount - 1 do
  begin
    with grdSelected do
    begin
      if TIVComponent(Objects[0, i]).IEN = OrderableIEN then
         cnt := cnt + 1;
    end;
  end;

  if cnt > 1 then
    Result := False
  else begin
    ShowMessage('Removing the last orderable item is not allowed!');
    Result := True;
  end;
end;

end.
