unit OrderComIntf;

interface

uses
  ComObj, ActiveX, AXCtrls, BcmaOrderCom_TLB, StdVcl, uCore, ORFn, ORNet, SysUtils, Classes, Dialogs;

type
  TBcmaOrder = class(TAutoObject, IBcmaOrder)
  protected
    procedure InitObjects(const PatientIEN: WideString; ProviderIEN: Int64;
      LocationIEN: Integer); safecall;
    function ConnectToServer(const ServerName: WideString;
      PortNumber: Int64): WordBool; safecall;
    function ActivateIVDialog(const DlgIEN, Solutions, Additives: WideString;
      var AnOrderID: WideString): WordBool; safecall;
    function ActivateUDDialog(const DlgIEN: WideString; OIIen: Int64;
      var AnOrderID: WideString): WordBool; safecall;
    function SignOrders(TimeOut: WordBool; var AnOrderList,
      AECode: WideString): WordBool; safecall;
    function SendOrders(const AnOrdersList, AECode: WideString): WordBool;
      safecall;
    function DcOrders(const AnOrderLst: WideString): WordBool; safecall;
    function BcmaUnlockPatient(const APatientIEN: WideString): WordBool;
      safecall;
    function BcmaUnlockOrders(const AnOrderList: WideString): WordBool;
      safecall;
  public
    destructor Destroy; override;
  end;

var
  Event: TOrderDelayEvent;
  OrderCount: Integer = 0;

implementation

uses ComServ,uBcmaOrder,rCore,rOrders;

procedure TBcmaOrder.InitObjects(const PatientIEN: WideString;
  ProviderIEN: Int64; LocationIEN: Integer);
begin
  User := TUser.Create;

  Patient := TPatient.Create;
  Patient.DFN := PatientIEN;

  Encounter := TEncounter.Create;
  Encounter.Provider := ProviderIEN;
  Encounter.Location := LocationIEN;
  Encounter.DateTime := FMNow;

  Event.EventType := 'C';
  Event.Specialty := 0;
  Event.Effective := 0;

  BcmaDrugIEN := 0;
  BcmaOrderID := '';
  BcmaSignedOrder    := '';
  BcmaCanceledOrder  := '';
  BcmaSignInfo       := '';
  if BcmaOrderChanges = nil then
    BcmaOrderChanges := TChanges.Create;
end;

function TBcmaOrder.ConnectToServer(const ServerName: WideString;
  PortNumber: Int64): WordBool;
begin
  EnsureBroker;
  with RPCBrokerV do
  begin
    Server := ServerName;
    if PortNumber > 0 then ListenerPort := PortNumber;
    DebugMode := False;
    AllowShared := True;
    Connected := True;
    CreateContext('OR BCMA ORDER COM');
  end;
  Result := RPCBrokerV.Connected;
end;

function TBcmaOrder.ActivateIVDialog(const DlgIEN, Solutions,
  Additives: WideString; var AnOrderID: WideString): WordBool;
begin
  if (Length(Solutions)< 1) then
  begin
    ShowMessage('There is no solution selected, you can not proceed for this IV order.');
    Result := False;
    Exit;
  end;
  BcmaSolutions := TStringList.Create;
  BcmaAdditives := TStringList.Create;
  PiecesToList(Solutions,'^',BcmaSolutions);
  PiecesToList(Additives,'^',BcmaAdditives);
  Result :=  ActivateOrderDialog(DlgIEN, Event);
  if length(BcmaOrderID)> 0 then
    AnOrderID := BcmaOrderID + '~' + IntToStr(Encounter.provider) + ';' + IntToStr(Encounter.Location);
  BcmaOrderID := '';
  BcmaSolutions.Clear;
  BcmaAdditives.Clear;
end;

function TBcmaOrder.ActivateUDDialog(const DlgIEN: WideString;
  OIIen: Int64; var AnOrderID: WideString): WordBool;
begin
  if not ReadyForNewOrder(Event) then
    Exit;
  BcmaDrugIen := OIIen;
  if BcmaDrugIen < 1 then
    Exit;
  Result := ActivateOrderDialog(DlgIEN, Event);
  if length(BcmaOrderID)> 0 then
    AnOrderID := BcmaOrderID + '~' + IntToStr(Encounter.provider) + ';' + IntToStr(Encounter.Location);
  BcmaOrderID := '';
end;

function TBcmaOrder.SignOrders(TimeOut: WordBool; var AnOrderList,
  AECode: WideString): WordBool;
begin
  if (BcmaOrderChanges <> nil) and (BcmaOrderChanges.Orders.Count > 0) then
  begin
    if not ToSignChanges(Timeout) then
    begin
      ANOrderList := '';
      AECode := '';
      BcmaSignedOrder    := '';
      BcmaCanceledOrder  := '';
      BcmaSignInfo       := '';
      Result := False;
    end else
    begin
     if (BcmaSignedOrder = '') and ( BcmaCanceledOrder = '') then
      AnOrderList := ''
     else if (BcmaSignedOrder <> '') and ( BcmaCanceledOrder = '') then
      AnOrderList := BcmaSignedOrder
     else if (BcmaSignedOrder = '') and ( BcmaCanceledOrder <> '') then
      AnOrderList := BcmaCanceledOrder
     else
      AnOrderList := BcmaSignedOrder + '^' + BcmaCanceledOrder;
     AECode := BcmaSignInfo;
     BcmaSignedOrder    := '';
     BcmaCanceledOrder  := '';
     BcmaSignInfo       := '';
     if BcmaOrderChanges.Count > 0 then
       BcmaOrderChanges.Clear;
     Result := True;
    end;
  end;
end;

function TBcmaOrder.SendOrders(const AnOrdersList,
  AECode: WideString): WordBool;
var
  OrderListBeSend: TStringList;
begin
  OrderListBeSend := TStringList.Create;
  PiecesToList(AnOrdersList,'^',OrderListBeSend);
  Result := SentOrdersToInpatientMeds(OrderListBeSend,AECode);
end;

function TBcmaOrder.DcOrders(const AnOrderLst: WideString): WordBool;
var
  TheOrderList: TStringList;
  AOrderID: string;
  AProviderIEN: int64;
  ALocationIEN:  integer;
  idx,count: integer;
begin
  Result := False;
  TheOrderList := TStringList.Create;
  PiecesToList(AnOrderLst,'^',TheOrderList);
  count := TheOrderList.Count;
  if count > 0 then
  begin
    for idx := 0 to count - 1 do
    begin
      AOrderID := Piece(TheOrderList[idx],'~',1);
      AProviderIEN := StrToInt64(Piece(Piece(TheOrderList[idx],'~',2),';',1));
      ALocationIEN  := StrToInt(Piece(Piece(TheOrderList[idx],'~',2),';',2));
      CallV('ORWDXA DC', [AOrderID, AProviderIEN, ALocationIEN, '']);
    end;
    Result := True;
  end;
  UnlockIfAble;
  if Assigned(BcmaOrderChanges) and (BcmaOrderChanges.count > 0 ) then
  begin
    BcmaOrderChanges.Clear;
    UnlockIfAble;
  end;
end;

function TBcmaOrder.BcmaUnlockPatient(
  const APatientIEN: WideString): WordBool;
begin
  result := UnLockThePatient(APatientIEN);
end;

destructor TBcmaOrder.Destroy;
begin
  if RPCBrokerV <> nil then
  begin
    if Patient <> nil then
       UnLockThePatient(Patient.DFN);
    RPCBrokerV.Connected := false;
  end;
  User := nil;
  Patient := nil;
  Changes := nil;
  BcmaDrugIEN := 0;
  BcmaOrderID := '';
  BcmaSignedOrder    := '';
  BcmaCanceledOrder  := '';
  BcmaSignInfo       := '';
  if BcmaOrderChanges <> nil then
    BcmaOrderChanges := nil;
  if BcmaSolutions <> nil then
    BcmaSolutions := nil;
  if BcmaAdditives <> nil then
    BcmaAdditives := nil;
  inherited;
end;

function TBcmaOrder.BcmaUnlockOrders(
  const AnOrderList: WideString): WordBool;
var
  OrderList: TStringList;
  i: integer;
begin
  Result := False;
  OrderList := TStringList.Create;
  PiecesToList(AnOrderList,'^',OrderList);
  for i := 0 to OrderList.Count - 1 do
    UnlockOrder(OrderList[i]);
  Result := True;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TBcmaOrder, Class_BcmaOrder,
    ciMultiInstance, tmApartment);
end.
